#!/bin/sh
#-----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.0 
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#-----------------------------------------------------------------------
#
# File Information: src/install/bin/conv_db2_jdbc.sh, twh_core, twh_1.1_fp1
#
# Revision: 1.9
#
# Date: 6/16/02 09:20:48
#-----------------------------------------------------------------------
#
# Primary Author: D. Robinson
#
#
# Description:  This script checks for the presence of
#               of DB2 and then runs the JDBC2.0 
#               conversion script that is shipped with
#               the DB2 product.
#
#
# Command Line Args: Presentation Service Installation
#                        Directory
#
# Required Config File: None
#
# Installation Logs (Default names):
#                       TWH install log = twh_db2_install.log
# 
# Required Env Vars: None
#
# Return Codes: 0=good, 1=error
#
# Interpreter Required: Bash 
#
# Message Types:
#                (I) - Informational
#                (W) - Warning
#                (F) - Fatal
#
# Installation steps required prior to running this script:
#                1) DB2 installed
#     
#######################################################

TRUE=1
FALSE=0
DB2Found=$FALSE
DB2InstallPath="/"
PathToJDBC="/"
PathToPSCD="/"
PathToDB2JavaZip="/"
OnNT=$FALSE
OnUnix=$FALSE

###############################################
# displays the usage of this script           #
###############################################
showUsage()
{
     echo "Program:   $PROGNAME"
     echo "Desc: Converts DB2 UDB Enterprise Edition "
     echo "      to run JDBC 2.0 "
     echo " "
     echo " Usage: conv_db2_jdbc PSInstallPath DB2InstallPath" 
}

# Display return code messages
display_return_code_messages()
{
     echo " "
     echo "      Return code messages for this program:"
     echo "      ======================================"
     echo "      0 = Success."
     echo "      1 = DB2 Installation directory not found."
     echo "      2 = DB2 supplied jdbc conversion scripts not found."
     echo "      3 = Execution of usejdbc2 script failed."
     echo "      4 = Copy of db2java.zip driver to PS directory failed."
     echo "      5 = Failed to restart the db2jds service."
     echo " "
}




###############################################
#         See if DB2 is there                 #
###############################################
checkForDB2()
{
     echo "      (I) Checking for DB2 installation"
     INTERP=`uname -s`

     if [ "$INTERP" = "Windows_NT" ]; then
        echo "      (I) conv_db2_jdbc.sh running for NT"
        OnNT=$TRUE
        DB2PATH="$DB2Param"
        PathToJDBC="$DB2PATH"/java12
        PathToDB2JavaZip="$DB2PATH"/java
     elif [ "$INTERP" = "AIX" ]; then
        echo "      (I) conv_db2_jdbc.sh running for AIX"
        OnNT=$FALSE
        OnUnix=$TRUE
        DB2User_Home=`lsuser -a home $DB2Param | cut -f2 -d"=" | cut -f1 -d" "` 
        DB2PATH="$DB2User_Home/sqllib"
        PathToJDBC="$DB2PATH/java12"
        PathToDB2JavaZip="$DB2PATH/java12"
     elif [ "$INTERP" = "SunOS" -o "$INTERP" = "Linux" ]; then
        echo "      (I) conv_db2_jdbc.sh running for SunOS or Linux"
        OnNT=$FALSE
        OnUnix=$TRUE
        DB2User_Home=`getent passwd $DB2Param | cut -f6 -d":"`
        DB2PATH="$DB2User_Home/sqllib"
        PathToJDBC="$DB2PATH/java12"
        PathToDB2JavaZip="$DB2PATH/java12"
     else
        echo "      (F) Invalid Interp Type"
        exit -1
     fi

     if [ "$INTERP" = "Windows_NT" ]; then     
          DB2PATH=`echo $DB2PATH | sed 's/\//\\\\/g'`
     fi

     echo "      (I) Exporting DB2PATH: $DB2PATH"
     export DB2PATH
     
     echo "      (I) Path to where usejdbc2 script is located: $PathToJDBC"
     echo "      (I) Path to where db2java.zip file is located: $PathToDB2JavaZip"
        
     # the path that we were passed should
     # be correct.  We grep for the file that
     # we want to run just to make sure it is
     # where we think it is 
    
     if [ ! -d "$PathToJDBC" ]; then
         echo "      (F) DB2 Install directory not found: $rc1"
         exit 1
     else
         echo "      (I) DB2 Install directory found."
     fi

     if [ $OnNT -eq $TRUE ]; then                      
     # look for an NT file
        if [ ! -f "$PathToJDBC/usejdbc2.bat" ]; then
           # it appears we are not NT
           # of course we could be on NT and had an error
           echo "      (F) Could not find NT version of usejdbc2 script."
           echo "      (F) The DB2 jdbc conversion file was not found."
           exit 2
        else
           echo "      (I) Located NT version of usejdbc2 script."
        fi
     elif [ $OnUnix -eq $TRUE ]; then
        if [ ! -f "$PathToJDBC/db2java.zip" ]; then
           echo "      (F) Could not find db2java.zip file."
           echo "      (F) The DB2 jdbc conversion file was not found."
           exit 2
        else
           echo "      (I) Found db2java.zip file."
        fi
     else
        echo "       (F) The DB2 jdbc conversion file was not found."
        echo "           Invalid configuration."
        exit 2   
     fi
         
     DB2Found=$TRUE 
}


###############################################
#         Run the DB2 script to set           #
#         up DB2                              #
###############################################
convertJDBCNT()
{
     echo "      (I) Starting DB2 conversion to JDBC 2.0"


     # first must stop a DB2 service that uses some
     # of the dlls we want to replace
     echo "      (I) Stopping DB2 db2jds service."
     blah=`net stop db2jds`
     rc=$?

     if [ $rc -ne 0 ]; then
       # may not be a fatal error....
       echo "      (W) Failed to stop db2jds"
     else
       echo "      (I) DB2 db2jds service stopped."
     fi 


     # run the db2 self extracting zip that converts 
     # db2 to jdbc2.0
     echo "      (I) Running $PathToJDBC\usejdbc2.bat"

     #issue the command to run the script   
     "$PathToJDBC"/usejdbc2.bat
     rc=$?

     if [ $rc -ne 0 ]; then
          echo "      (F) Execution of usejdbc2.bat failed: $rc"
          exit 3
     else
          echo "      (I) Execution of usejdbc2.bat completed successfully."
     fi

     # now copy the db2java.zip file that was unpacked to the ps 
     # cd directory
     echo "      (I) Copy from: $PathToDB2JavaZip/db2java.zip"
     echo "      (I) Copy to  : $DestDir/."
     cp "$PathToDB2JavaZip"/db2java.zip  "$DestDir"/.
     rc=$?
     if [ $rc -ne 0 ]; then
          echo "      (F) Copy of db2java.zip to Presentation Services failed: $rc"
          exit 4
     else
          echo "      (I) Copy of db2java.zip to Presentation Services completed successfully."
     fi

     # now start the db2jds applet listener for DB2 
     # we stopped this to do our conversion
     blah=`net start db2jds`
     rc=$?
     if [ $rc -ne 0 ]; then
          echo "      (F) Failed to restart DB2 db2jds service."
          exit 5
     else
          echo "      (I) DB2 db2jds service restarted successfully."
     fi 
}


###############################################
#         Run the DB2 script to set           #
#         up DB2                              #
###############################################
convertJDBCUnix()
{
     echo "      (I) Unix conversion routine called."
     
     echo "      (I) Copy from: $PathToDB2JavaZip/db2java.zip"
     cp "$PathToJDBC/db2java.zip"  "$DestDir/."
     rc=$?
     if [ $rc -ne 0 ]; then
          echo "      (F) Copy of db2java.zip to destination directory $DestDir failed: $rc" 
          exit 4
     else
          echo "      (I) Copy of db2java.zip to destination directory completed successfully." 
     fi
     
}



###############################################
#                MAIN                         #
###############################################

# only one parameter for this script
# that parameter is the ps install directory

if [ $# -eq 2 ]; then

     #shows the number of parameters
     numParams=$#
     echo "      (I) Number of parameters received is: $numParams"

     #shows the actual parameters entered
     DestDir="$1"
     DB2Param="$2"
     echo "      (I) DestDir is: $DestDir" 
     echo "      (I) DB2Param is: $DB2Param"
     
     if [ ! -d "$DestDir" ]; then
         echo "      (I) DestDir is not found.  Creating..."
         mkdir -p "$DestDir"
     fi


     # if db2 is not there we can't do 
     # our work
     checkForDB2;

     if [ $DB2Found -eq $TRUE ]; then
         if [ $OnNT -eq $TRUE ]; then
            convertJDBCNT;
         elif [ $OnUnix -eq $TRUE ]; then
            convertJDBCUnix;
         fi
     fi

     echo "      (I) Conversion of DB2 to JDBC 2.0 completed successfully"
     exit 0
else
     showUsage;
fi


