#!/bin/sh
#----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.1 
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#----------------------------------------------------------------------
#
# File Information: src/install/bin/twh_gen_routines.sh, twh_core, twh_1.1_ode
#
# Revision: 1.36
#
# Date: 5/3/02 16:06:34
#----------------------------------------------------------------------
#
# Primary Author: D. Ledden
#
#
# Desc:	This program will include the generic routines to be used
#		by other scripts.  THIS IS NOT A "RUNABLE" SCRIPT.  These
#		routines will be used by the Core install, the App install,
#		and the App Afterscript
#
# Required Env Vars:
#	None.
#
# Return Codes:
#	* See display_return_code_messages function below
#	* Return codes 3-70 reserved for the gen routines
#	* Return code 255 reserved for application afterscript errors.
#
#
# Interpreter Required:
#	NT/2000 = BASH
#	Unix	= SH (Bourne Shell)
#
#
# Message Types:
#	(I) - Informational
#	(W) - Warning
#	(F) - Fatal
#
#
# Notes: All routines in this file will start with gr_ which
#	   stands for Generic Routine to denote where they came from.
#
# Contexts: These routines will have two contexts to run with
#		provided by the scripts that use this file.
#		TWH_CORE_CONTEXT = Running from Core script
#		TWH_APP_CONTEXT  = Running from App script
#
#		This allows these routines to determine who is currently
#		using them and how it should handle things.
#
#########################################################################
#
###############################################################
################ Custom Functions Section #####################
###############################################################

# Display the program header
gr_display_generic_header()
{
app_verbiage=$1
if [ -z "$app_verbiage" ]; then app_verbiage=""; fi

echo ""
echo "    ========================================================"
echo "    Tivoli Enterprise Data Warehouse (TEDW) - IBM software"
echo ""
echo "    $app_verbiage"
echo "    Version 1.1.1"		# might consider runtime var for this
echo "    All rights reserved"
echo ""
echo "    Please read the documentation regarding the use"
echo "    and licensing of DB2 with this product."
echo "    ========================================================"
echo ""

return 0;
}


###


# Process the command line arguments
# Args:
# 	$1 = argument number that must be met
#	$2 = Number that was passed found
# Globals:
#	TWH_CONFIG_FILE 		= The file and path of the config file
#	TWH_CONFIG_FILE_NAME 	= The file name only of the config file
# Notes:
#	Requires a function called display_help and global 
#     DISPLAY_HELP_ACTIVE to display the help on errors
# 
gr_process_args()
{
required_args=$1
args_found=$2
if [ -z $required_args ]; then required_args=0; fi
if [ -z $args_found ]; then args_found=0; fi


# Look for the config file option flag first
if [ -n "$TWH_CONFIG_FILE" ]; then
  TWH_CONFIG_NT_FILE=`echo "$TWH_CONFIG_FILE" | sed 's/\\\\/\//g'`
  TWH_CONFIG_FILE_NAME=`basename "$TWH_CONFIG_NT_FILE"`
  args_found=`expr $args_found + 1`

  # Do we really have the file
  if [ ! -r "$TWH_CONFIG_FILE" ]; then
    echo "      (F) CDWIC0003E Unable to read/locate config file ($TWH_CONFIG_FILE)"
    echo "          Must specify the full path if the file is not in"
    echo "          the working/current directory"
    echo ""
    exit 3
  fi
fi


# Did we get all required arguments needed?
if [ $required_args -gt $args_found ]; then
  echo ""
  echo "      (F) CDWIC0004E Not all required command line options were supplied."
  if [ -z "$DISPLAY_HELP_ACTIVE" ]; then
    echo "          Displaying Help"
    echo ""
    display_help;
  fi
  exit 4
fi


return 0;
}


###


########################################################################
# Config file reader.
# Basic rules:
# 1 - remove all whitespace
# 2 - '#' is the comment character. Ignore following text.
# 3 - properties take the value of this=that exactly
# 4 - Must account for all props in file - none can be missing
#     (corrupt file)
# 5 - Special characters not allowed (i.e.  *, ", ')
########################################################################

# Validate all config options
gr_validate_config_opts()
{

# Validate all properties are here (Shared/Generic)
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_TOPDIR not found in config file"
  exit 5
elif [ -z "$TWH_VERSION" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_VERSION not found in config file"
  exit 5
elif [ -z "$TMPDIR" ]; then
  echo "      (F) CDWIC0005E Error.  Property TMPDIR not found in config file"
  exit 5
elif [ -z "$TWH_KEEP_TMP_FILES" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_KEEP_TMP_FILES not found in config file"
  exit 5
elif [ -z "$DB2_INST_DIR" ]; then
  echo "      (F) CDWIC0005E Error.  Property DB2_INST_DIR not found in config file"
  exit 5
elif [ -z "$DB2_INSTALL_LOG" ]; then
  echo "      (F) CDWIC0005E Error.  Property DB2_INSTALL_LOG not found in config file"
  exit 5
elif [ -z "$DB2USER" ]; then
  echo "      (F) CDWIC0005E Error.  Property DB2USER not found in config file"
  exit 5
elif [ -z "$DB2PASS" ]; then
  echo "      (F) CDWIC0005E Error.  Property DB2PASS not found in config file"
  exit 5
elif [ -z "$TWH_CTRL_DB" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_CTRL_DB not found in config file"
  exit 5
elif [ -z "$TWH_CDW_DB" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_CDW_DB not found in config file"
  exit 5
elif [ -z "$TWH_DM_DB" ]; then
  echo "      (F) CDWIC0005E Error.  Property TWH_DM_DB not found in config file"
  exit 5
fi


# Now validate by context
if [ -n "$TWH_CORE_CONTEXT" -o -n "$TWH_CORE_PATCH_CONTEXT" ]; then
  if [ -z "$COPT_CDW_DB2USER" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CDW_DB2USER not found in config file"
    exit 5
  elif [ -z "$COPT_CDW_DB2PASS" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CDW_DB2PASS not found in config file"
    exit 5
  elif [ -z "$COPT_CDW_DB2HOST" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CDW_DB2HOST not found in config file"
    exit 5
  elif [ -z "$COPT_CDW_DB2PORT" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CDW_DB2PORT not found in config file"
    exit 5
  elif [ -z "$COPT_MART_DB2USER" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_MART_DB2USER not found in config file"
    exit 5
  elif [ -z "$COPT_MART_DB2PASS" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_MART_DB2PASS not found in config file"
    exit 5
  elif [ -z "$COPT_MART_DB2HOST" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_MART_DB2HOST not found in config file"
    exit 5
  elif [ -z "$COPT_MART_DB2PORT" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_MART_DB2PORT not found in config file"
    exit 5
  elif [ -z "$COPT_CTRL_DB2HOST" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CTRL_DB2HOST not found in config file"
    exit 5
  elif [ -z "$COPT_CTRL_DB2PORT" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CTRL_DB2PORT not found in config file"
    exit 5
  elif [ -z "$COPT_CTRL_DB2USER" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CTRL_DB2USER not found in config file"
    exit 5
  elif [ -z "$COPT_CTRL_DB2PASS" ]; then
    echo "      (F) CDWIC0005E Error.  Property COPT_CTRL_DB2PASS not found in config file"
    exit 5
  elif [ -z "$LOCAL_HOSTNAME" ]; then
    echo "      (F) CDWIC0005E Error.  Property LOCAL_HOSTNAME not found in config file"
    exit 5
  fi
# Application context
elif [ -n "$TWH_APP_CONTEXT" ]; then
  if [ -z "$APP_MEDIA_DIR" ]; then
    echo "      (F) CDWIC0005E Error.  Property APP_MEDIA_DIR not found in config file"
    exit 5
  elif [ -z "$APP_INSTALL_LIST_FILE" ]; then
    echo "      (F) CDWIC0005E Error.  Property APP_INSTALL_LIST_FILE not found in config file"
    exit 5
  elif [ -z "$APP_INSTALL_PROPS_FILE" ]; then
    echo "      (F) CDWIC0005E Error.  Property APP_INSTALL_PROPS_FILE not found in config file"
    exit 5
  elif [ -z "$PS_HOME" ]; then
    echo "      (F) CDWIC0005E Error.  Property PS_HOME not found in config file"
    exit 5
  fi
# Application Deinstall context
elif [ -n "$TWH_APP_DEINST_CONTEXT" ]; then
  if [ -z "$APPLICATION_TO_DELETE" ]; then
    echo "      (F) CDWIC0005E Error.  Property APPLICATION_TO_DELETE not found in config file"
    exit 5
  fi
elif [ -n "$TWH_CORE_PATCH_CONTEXT" ]; then
  if [ -z "$PS_HOME" ]; then
    echo "      (F) CDWIC0005E Error.  Property PS_HOME not found in config file"
    exit 5
  fi
  if [ -z "$PATCH_MEDIA_DIR" ]; then
    echo "      (F) CDWIC0005E Error.  Property PATCH_MEDIA_DIR not found in config file"
    exit 5
  fi
elif [ -n "$TWH_APP_PATCH_CONTEXT" ]; then
  if [ -z "$APP_MEDIA_DIR" ]; then
    echo "      (F) CDWIC0005E Error.  Property APP_MEDIA_DIR not found in config file"
    exit 5
  elif [ -z "$APP_INSTALL_PROPS_FILE" ]; then
    echo "      (F) CDWIC0005E Error.  Property APP_INSTALL_PROPS_FILE not found in config file"
    exit 5
  elif [ -z "$PS_HOME" ]; then
    echo "      (F) CDWIC0005E Error.  Property PS_HOME not found in config file"
    exit 5
  fi
else
  echo "      (F) CDWIC0006E No context specified for Generic Routine"
  exit 6
fi


### Basic tests on the config options

# check tmp dir
gr_test_tmpdir;

# ??? Add more tests on other options here


echo "      (I) Validated all Config options"
return 0;
}


###


# Print any config read errors
gr_print_conf_read_error()
{
echo "      (F) Config file parsing error.  Expected form 'property=value', but"
echo "          found value ($1) on line $line_count"
echo "          Only two forms accepted:"
echo "            1).  # = Comment"
echo "            2).  property takes the form 'property=value'"
echo ""
echo "            Corrupt config file - Exiting!"
echo ""

return 0;
}


###


# Read the config file and validate the properties supplied
gr_read_config_file()
{
echo "==> Reading Config File ($TWH_CONFIG_FILE)"


# Preset vars
line_count=0

# Open our config file (Need to assist NT BASH here)
if [ $UNIX_INTERP -eq 1 ]; then
  exec 4< "$TWH_CONFIG_FILE"
else
  exec < "$TWH_CONFIG_FILE"		# For NT
fi


# Parse the config file
while :
do
#  read cur_line
  # Read the config file (Need to assist NT BASH here)
  if [ $UNIX_INTERP -eq 1 ]; then
    read cur_line <& 4
  else
    read cur_line
  fi

  # Did we hit the end of the file?
  if [ ! $? -eq 0 ]; then break; fi

  line_count=`expr $line_count + 1`

  # clean whitespace and look for comment char
  cleaned_line=`echo "$cur_line" | sed "s/        //g" `
  echo "$cleaned_line" | grep -i "^#" > /dev/null
  if [ $? -eq 0  -o ! "$cleaned_line" ]; then
    continue
  fi

  # Now parse for properties and validate them
  echo "$cleaned_line" | grep "=" > /dev/null
  if [ ! $? -eq 0 ]; then
    gr_print_conf_read_error $cleaned_line ;
    exit 5
  fi

  # Find the property tag and value (upcase the tag)
  property_line=`echo "$cleaned_line" | sed "s/#.*//" `
  prop_tag=`echo "$property_line" | sed "s/=.*//" | tr '[a-z]' '[A-Z]' `
  prop_val=`echo "$property_line" | sed "s/.*=//" `


  # Determine the property tag and set the property
  case $prop_tag in

    ###############
    # Common Tags #
    ###############

    #
    TWH_TOPDIR)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_TOPDIR="$prop_val";
        export TWH_TOPDIR
        ;;

    #
    TWH_VERSION)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_VERSION="$prop_val"
        export TWH_VERSION
        ;;

    #
    TMPDIR)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TMPDIR="$prop_val"
        export TMPDIR
        ;;

    # Will assign to TWH_KEEP_TMP_FILES_TRUE for easy tests
    TWH_KEEP_TMP_FILES)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
	  TWH_KEEP_TMP_FILES="$prop_val"		# Only used to verify exists

	  # convert to upper - if TRUE then define
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          TWH_KEEP_TMP_FILES_TRUE="$prop_val"
          echo "      (I) Keeping tmp files"
        fi
        export TWH_KEEP_TMP_FILES
        ;;

    #
    DB2_INST_DIR)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        DB2_INST_DIR="$prop_val"
        export DB2_INST_DIR
        ;;

    #
    DB2_INSTALL_LOG)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        DB2_INSTALL_LOG="$prop_val"
        export DB2_INSTALL_LOG
        ;;

    #
    DB2USER)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        DB2USER="$prop_val"
        export DB2USER
        ;;

    #
    DB2PASS)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        if [ -n "$BYPASS_PASSWORD_REQUIRED" ]; then
          DB2PASS="not_required"
        else
          DB2PASS="$prop_val"
        fi
        export DB2PASS
        ;;

    #
    TWH_CTRL_DB)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_DB="$prop_val"
        # Clean up extra spaces on this.
        TWH_CTRL_DB=`echo $TWH_CTRL_DB | sed "s/        //g" `

        export TWH_CTRL_DB
        ;;

    #
    TWH_CDW_DB)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_DB="$prop_val"

        # Clean up extra spaces on this.
        TWH_CDW_DB=`echo $TWH_CDW_DB | sed "s/        //g" `

        export TWH_CDW_DB
        ;;

    #
    TWH_DM_DB)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_DM_DB="$prop_val"

        # Clean up extra spaces on this.
        TWH_DM_DB=`echo $TWH_DM_DB | sed "s/        //g" `

        export TWH_DM_DB
        ;;

    LOCAL_HOSTNAME)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        LOCAL_HOSTNAME="$prop_val"
        export LOCAL_HOSTNAME
        ;;

    # Only Supplied non-null when custom install

    #
    COPT_CDW_DB2HOST)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CDW_DB2HOST="$prop_val"
        export COPT_CDW_DB2HOST
        ;;

    #
    COPT_CDW_DB2PORT)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CDW_DB2PORT="$prop_val"
        export COPT_CDW_DB2PORT
        ;;

    #
    COPT_CDW_DB2USER)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CDW_DB2USER="$prop_val"
        export COPT_CDW_DB2USER
        ;;

    #
    COPT_CDW_DB2PASS)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        if [ -n "$BYPASS_PASSWORD_REQUIRED" ]; then
          COPT_CDW_DB2PASS="not_required"
        else
          COPT_CDW_DB2PASS="$prop_val"
        fi
        export COPT_CDW_DB2PASS
        ;;


    #
    COPT_MART_DB2HOST)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_MART_DB2HOST="$prop_val"
        export COPT_MART_DB2HOST
        ;;

    #
    COPT_MART_DB2PORT)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_MART_DB2PORT="$prop_val"
        export COPT_MART_DB2PORT
        ;;

    #
    COPT_MART_DB2USER)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_MART_DB2USER="$prop_val"
        export COPT_MART_DB2USER
        ;;

    #
    COPT_MART_DB2PASS)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        if [ -n "$BYPASS_PASSWORD_REQUIRED" ]; then
          COPT_MART_DB2PASS="not_required"
        else
          COPT_MART_DB2PASS="$prop_val"
        fi
        export COPT_MART_DB2PASS
        ;;

	
    #
    COPT_CTRL_DB2HOST)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CTRL_DB2HOST="$prop_val"
        export COPT_CTRL_DB2HOST
        ;;

    #
    COPT_CTRL_DB2PORT)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CTRL_DB2PORT="$prop_val"
        export COPT_CTRL_DB2PORT
        ;;

    #
    COPT_CTRL_DB2USER)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        COPT_CTRL_DB2USER="$prop_val"
        export COPT_CTRL_DB2USER
        ;;

    #
    COPT_CTRL_DB2PASS)
	if [ $DEBUG ]; then echo "$prop_tag found"; fi
        if [ -n "$BYPASS_PASSWORD_REQUIRED" ]; then
          COPT_CTRL_DB2PASS="not_required"
        else
          COPT_CTRL_DB2PASS="$prop_val"
        fi
        export COPT_CTRL_DB2PASS
        ;;


    ###############
    # Core Tags   #
    ###############



    ###############
    # App Tags    #
    ###############
    #
    APP_MEDIA_DIR)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_MEDIA_DIR="$prop_val"
        export APP_MEDIA_DIR
        ;;

    #
    APP_INSTALL_LIST_FILE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_INSTALL_LIST_FILE="$prop_val"
        export APP_INSTALL_LIST_FILE
        ;; 

    #
    APP_INSTALL_PROPS_FILE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_INSTALL_PROPS_FILE="$prop_val"
        export APP_INSTALL_PROPS_FILE
        ;; 

    #
    PS_HOME)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PS_HOME="$prop_val"
        export PS_HOME
        ;;

    #########################
    # App Deinstall Tags    #
    #########################
    #
    APPLICATION_TO_DELETE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APPLICATION_TO_DELETE="$prop_val"
        export APPLICATION_TO_DELETE
        ;;

    #########################
    # Core Patch Tags       #
    #########################
    #
    PS_HOME)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PS_HOME="$prop_val"
        export PS_HOME
        ;;

    PATCH_MEDIA_DIR)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PATCH_MEDIA_DIR="$prop_val"
        export PATCH_MEDIA_DIR
        ;;

    #########################
    # App Patch Tags        #
    #########################
    #
    PS_HOME)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PS_HOME="$prop_val"
        export PS_HOME
        ;;

    PATCH_MEDIA_DIR)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PATCH_MEDIA_DIR="$prop_val"
        export PATCH_MEDIA_DIR
        ;;

    # Default - is an error
    *)
        echo "      (F) CDWIC0007E Illegal option found."
        echo "          Illegal value is ($prop_tag)"
        exit 7
        ;;
    esac
  done


# Now close the config file (For BASH with NT leave open for causes bug)
if [ $UNIX_INTERP -eq 1 ]; then
  exec 4<&-
fi


# Detect any missing properties
gr_validate_config_opts;

echo "      (I) Completed reading Config File"
return 0;
}


###


# Set the program configurable variables
gr_set_program_vars()
{
PATH="$PATH:$DB2_INST_DIR/BIN:$DB2_INST_DIR/FUNCTION:$DB2_INST_DIR/HELP"
export PATH

# Program Configurable Vars
# DB2
DB2CMD="db2cmd /w /c /i"
#DB2CMD="bin/db2cmd /w /c /i"
DB2CARGS="/w /c /i"					# The DB2 cmd line
DB2PROG="db2"						# DB2 interface command
DB2EXEC="$DB2CMD $DB2PROG"				# Execute db2 command
DB2EXEC_LOG="$DB2EXEC -z \"$TMPDIR/$DB2_INSTALL_LOG\""	# Log file
DB2EXEC_LOG_DDL="$DB2EXEC_LOG -tvf"			# Log and run DDL
DB2REGI="$DB2CMD iwh2regi"				# DWC Registry command
DB2IMP="$DB2CMD iwh2imp2"				# DWC WH Import command

# Program specific
TWH_CTRL_SCHEMA=IWH	# Standard schema name (Don't change)
CORE_VERSION_FILE="$TWH_TOPDIR/.twh_core_v$TWH_VERSION.marker"
CORE_INSTALL_TYPICAL_FILE="$TWH_TOPDIR/.twh_core_typical.marker"
CORE_INSTALL_CUSTOM_FILE="$TWH_TOPDIR/.twh_core_custom.marker"
TWH_CORE_COMP_FILE="$TWH_TOPDIR/install/logs/twh_core_comp_list.cfg"
VWSPATH="$DB2_INST_DIR"; export VWSPATH 	# For .tag files imports


TRUE=1
FALSE=0
WARNING_FLAG=$FALSE			# if warnings found
ERROR_CONTINUE=$FALSE			# Continue on Errors found

# Interp group types
CLASS_WIN=1					# Can be win only type (DWC admin)
CLASS_TYPE_1=2				# Can be win, sol, aix (CDW, MART)
CLASS_TYPE_2=3				# Can be win, sol, aix, linux (RPI)

# Custom Options
COPT_CTRL_TYPE=0
COPT_CDW_TYPE=1
COPT_MART_TYPE=2
COPT_RPI_TYPE=3


# node_alias_type
NO_ALIAS_TYPE=0
CTRL_ALIAS_TYPE=1
CDW_ALIAS_TYPE=2
MART_ALIAS_TYPE=3


# Connection types
CONN_TYPE_CONNECT=1
CONN_TYPE_ATTACH=2

# Globalization settings (for create db)
DB2CODEPAGE=1208; export DB2CODEPAGE		# UTF-8 Codepage

# PS server action
START_PS=1
STOP_PS=2
RECYCLE_PS=3


return 0;
}


###


# Set the Interp
gr_set_interp()
{
WIN=W32-IX86
SOL=SOLARIS2
AIX=AIX4-R1
HP=HPUX10
LINUX=LINUX

echo "==> Setting Interp"
UNIX_INTERP=notset
supported_types="$WIN, $SOL, $AIX, $LINUX"

if [ -z "$INTERP" ]; then
  INTERP=notset
  unix_version=`uname -s | tr '[a-z]' '[A-Z]'`

  if [ $unix_version = "SUNOS" ]; then
    INTERP=$SOL
    UNIX_INTERP=1
  elif [ $unix_version = "AIX" ]; then
    INTERP=$AIX
    UNIX_INTERP=1
  elif [ $unix_version = "LINUX" ]; then
    INTERP=$LINUX
    UNIX_INTERP=1
  elif [ $unix_version = "HP-UX" ]; then
    INTERP=$HP
    UNIX_INTERP=1

    # HP not supported for now
    echo "      (F) CDWIC0008E Interp (HP-UX) not currently supported"
    echo "          TEDW supported types:"
    echo "            $supported_types"

    exit 8
  elif [ $unix_version = "WINDOWS_NT" ]; then
    INTERP=$WIN
    UNIX_INTERP=0
  else
    echo "      (F) CDWIC0008E Interp ($unix_version) not supported"
    echo "          TEDW supported types:"
    echo "            $supported_types"

    exit 8
  fi

  export INTERP
else
  INTERP=`echo $INTERP | tr '[a-z]' '[A-Z]'`
  if [ "$INTERP" = "$SOL" -o "$INTERP" = "$AIX" ]; then
    UNIX_INTERP=1
  elif [ "$INTERP" = "$WIN" ]; then
    UNIX_INTERP=0
  else
    echo "      (F) CDWIC0008E Interp ($INTERP) not supported"
    echo "          TEDW supported types:"
    echo "            $supported_types"

    exit 8
  fi
fi


# Set  output
if [ "$INTERP" = "$WIN" ]; then
   DUMMY_OUT="nul"; else DUMMY_OUT="/dev/null";
fi


echo "      (I) Interp is ($INTERP)"
return 0;
}


###


# Check to see if we have enough free disk space
# To call:  gr_check_disk_usage $1 $2;
#           $1 = requirement in k's
#           $2 = directory to check
#
gr_check_disk_usage()
{
disk_min=$1
dir_to_check="$2"
space_error="      (F) Not enough available at ($dir_to_check)"
one_k=1024

# Determine OS and capacity
if [ "$INTERP" = "$SOL" ]; then
  disk_usage=`df -b $dir_to_check | awk '{print $2}'  | grep -v 'avail'`
elif [ "$INTERP" = "$AIX" ]; then
  disk_usage=`df -k $dir_to_check | awk '{print $3}' | grep -v 'Free'`
elif [ "$INTERP" = "$LINUX" ]; then
  disk_usage=`df -k $dir_to_check | awk '{print $4}' | grep -v 'Available'`
elif [ "$INTERP" = "$HP" ]; then
  disk_usage=`df -b $dir_2_check | awk '{print $5}'`
elif [ "$INTERP" = "$WIN" ]; then
  disk_usage=`ntfsinfo -a "$dir_to_check"`

  # must convert from bytes to kilobytes
  if [ $disk_usage -lt 1 ]; then
     echo "$space_error"
     exit 9
  fi
else
   echo "      (F) CDWIC0009E Can't determine disk usage for $INTERP"
   exit 9
fi

# calculate avail space
if [ $disk_usage -lt $disk_min ]; then
   echo "$space_error"
   echo "          Space Avail  = "$disk_usage"k"
   echo "          Space Needed = "$disk_min"k"
   exit 9
fi

echo "      (I) Space is available for ($dir_to_check)"

return 0;
}


###


# Check to see if they are running with the correct interp class for
# this component request.
# To call:  gr_check_interp_by_class $1;
#           $1 = Interp class type
#
gr_check_interp_by_class()
{
requested_class=$1
echo "      (I) Checking the component interp dependencies"


# Validate input
if [ -z "$requested_class" ]; then
  echo "      (F) CDWIC0025E No interp class supplied to function.  Dev error"
  exit 25
fi

# Validate interps that this can run on
if [ $requested_class -eq $CLASS_WIN ]; then
  # Windows only
  if [ "$INTERP" != "$WIN" ]; then
    echo "      (F) CDWIC0026E You are only allowed to execute this option on a"
    echo "          Windows based platform."
    echo "          Current OS Interp is ($INTERP)"
    exit 26
  fi
elif [ $requested_class -eq $CLASS_TYPE_1 ]; then
  # Win, Sol, AIX
  if [ "$INTERP" != "$WIN" -a "$INTERP" != "$SOL" -a "$INTERP" != "$AIX" ]; then
    echo "      (F) CDWIC0027E You are only allowed to execute this option on a"
    echo "          Windows, Solaris, or AIX based platform"
    echo "          Current OS Interp is ($INTERP)"
    exit 27
  fi
elif [ $requested_class -eq $CLASS_TYPE_2 ]; then
  if [ "$INTERP" != "$WIN" -a "$INTERP" != "$SOL" -a "$INTERP" != "$AIX" -a "$INTERP" != "$LINUX" ]; then
    echo "      (F) CDWIC0028E You are only allowed to execute this option on a"
    echo "          Windows, Solaris, AIX, or Linux based platform"
    echo "          Current OS Interp is ($INTERP)"
    exit 28
  fi
else
  echo "      (F) CDWIC0029E Unknown interp class.  Dev error"
  exit 29
fi

echo "      (I) completed checking the component interp dependencies"
return 0;
}


###


# Debug Mode
gr_set_debug_mode()
{
echo "==> Setting Debug Mode"

if [ -n "$DEBUG" ]; then
   set -x
   echo "      (I) Debug Mode Enabled"
else
   echo "      (I) Debug Disabled"
fi

return 0;
}

###


# Test tmpdir
gr_test_tmpdir()
{
if [ ! -d "$TMPDIR" ]; then
  mkdir "$TMPDIR"
fi

testfile=test_twh.txt

blah=`touch "$TMPDIR/$testfile" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIC0010E Temp directory ($TMPDIR) not accessable."
  echo "          Needed for installation."
  exit 10
else
  blah=`rm "$TMPDIR/$testfile" 2>&1`
fi

echo "      (I) Temp Directory is ($TMPDIR) and is accessable"

#gr_check_disk_usage 20000 "$TMPDIR";	# 20 meg ballpark needed
gr_check_disk_usage 10000 "$TMPDIR";	# 20 meg ballpark needed


return 0;
}


###


# Extend our path for compatibility in the program
gr_extend_path()
{
echo "==> Checking file path"
# Only for unix
if [ "$UNIX_INTERP" -ne 1 ]; then
  echo "      (I) No need to extend file path"
  return 0;
fi

blah=`type whoami 2>&1`
if [ $? -eq 0 ]; then return 0; fi

if [ ! -d "/usr/ucb" -a "$INTERP" != "$HP" ]; then
  echo "      (F) CDWIC0011E Directory not found ("/usr/ucb")"
  echo "            The Unix ucb tools are needed to use this installer"
  exit 11
fi
PATH="$PATH:/usr/ucb"

echo "      (I) Extending Unix tools path to include (/usr/ucb)"
return 0;
}


###


# Check that we have the right user
gr_check_user()
{
echo "==> Verifying user"

if [ "$UNIX_INTERP" -eq 1 ]; then
  ROOT_USER=0
  real_user=`id | sed "s/uid=//" | sed "s/(.*//" `
  if [ $real_user -ne $ROOT_USER ]; then
    echo "      (F) CDWIC0012E Invalid user id, must be root user to install this program"
    exit 12
  fi
else
  # NT OS  (Should use C code call to determine user)

  ROOT_USER="ADMINISTRATOR"
  my_id=`whoami | tr '[a-z]' '[A-Z]'`
#  if [ $my_id != "$ROOT_USER" ]; then
#    echo "      (F) CDWIC0012E Invalid user id, Must be $ROOT_USER to install this program"
#    exit 12
#  fi
fi


echo "      (I) Valid user found"
return 0;
}


###


#
gr_check_version()
{
echo "==> Checking Release Version"

# Prevent from rerunning
if [ -r "$CORE_VERSION_FILE" ]; then
  if [ $CUSTOM_INSTALL -eq $TRUE ]; then
    if [ -r "$TWH_CORE_COMP_FILE" ]; then
      if [ $ADMIN_CHECKED -eq $TRUE ]; then
        blah=`grep "TWH_CTRL_COMP" "$TWH_CORE_COMP_FILE" | cut -d"=" -f2`
        if [ "$blah" = "TRUE" ]; then
          echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system."
          echo "          Control Server custom install method has already been run." 
          echo "          To prevent destroying an existing installation,"
          echo "          you cannot reinstall.  You must explicitly"
          echo "          uninstall first."
          echo "          Exiting..."
          exit 13
        fi
      fi
      if [ $RPI_CHECKED -eq $TRUE ]; then
        blah=`grep "TWH_RPI_COMP" "$TWH_CORE_COMP_FILE" | cut -d"=" -f2`
        if [ "$blah" = "TRUE" ]; then
          echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system."
          echo "          RPI Server custom install method has already been run." 
          echo "          To prevent destroying an existing installation,"
          echo "          you cannot reinstall.  You must explicitly"
          echo "          uninstall first."
          echo "          Exiting..."
          exit 13
        fi
      fi
      if [ $CDW_CHECKED -eq $TRUE ]; then
        blah=`grep "TWH_CDW_COMP" "$TWH_CORE_COMP_FILE" | cut -d"=" -f2`
        if [ "$blah" = "TRUE" ]; then
          echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system."
          echo "          CDW Server custom install method has already been run." 
          echo "          To prevent destroying an existing installation,"
          echo "          you cannot reinstall.  You must explicitly"
          echo "          uninstall first."
          echo "          Exiting..."
          exit 13
        fi
      fi
      if [ $MART_CHECKED -eq $TRUE ]; then
        blah=`grep "TWH_MART_COMP" "$TWH_CORE_COMP_FILE" | cut -d"=" -f2`
        if [ "$blah" = "TRUE" ]; then
          echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system."
          echo "          Data Mart custom install method has already been run." 
          echo "          To prevent destroying an existing installation,"
          echo "          you cannot reinstall.  You must explicitly"
          echo "          uninstall first."
          echo "          Exiting..."
          exit 13
        fi
      fi
    else
      echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system."
      echo "          To prevent destroying an existing installation,"
      echo "          you cannot reinstall.  You must explicitly"
      echo "          uninstall first."
      echo "          Exiting..."
      exit 13
    fi
  else 
    echo "      (F) CDWIC0013E TEDW Version ($TWH_VERSION) is already detected on this system." 
    echo "          To prevent destroying an existing installation,"
    echo "          you cannot reinstall.  You must explicitly"
    echo "          uninstall first."
    echo "          Exiting..."
    exit 13
  fi  
fi

# Prevent them from trying to complete a install switching from 
# Typical to Custom.  Will cause problems with remote DB's
if [ -r "$CORE_INSTALL_TYPICAL_FILE" -a $CUSTOM_INSTALL -eq $TRUE ]; then
  echo "      (F) CDWIC0013E You cannot switch installation types without fully"
  echo "          deinstalling this product - which includes removing"
  echo "          all of the databases created by this product."
  echo "          You have initially installed the DB's through"
  echo "          a \"Typical\" installation, but are trying a"
  echo "          \"Custom\" install."
  echo "          Exiting..."
  exit 13
elif [ -r "$CORE_INSTALL_CUSTOM_FILE" -a $CUSTOM_INSTALL -eq $FALSE ]; then
  echo "      (F) CDWIC0013E You cannot switch installation types without fully"
  echo "          deinstalling this product - which includes removing"
  echo "          all of the databases created by this product."
  echo "          You have initially installed the DB's through"
  echo "          a \"Custom\" installation, but are trying a"
  echo "          \"Typical\" install."
  echo "          Exiting..."
  exit 13
fi

echo "      (I) Version ($TWH_VERSION) is acceptable"
return 0;
}


###


#
gr_make_directories()
{
# Add a logs directory
if [ ! -d "$TWH_TOPDIR"/install/logs  ]; then
  blah=`mkdir -p "$TWH_TOPDIR"/install/logs `
  if [ $? -ne 0 ]; then
    echo "      (F) CDWIC0014E Failed to create the (install/logs) directory"
    exit 14
  fi
fi


return 0;
}


###


# Generic database creation function
# To call:  gr_create_database $1;
#           $1 = DB name to create
#           $2 = user id
#           $3 = password
#		$4 = Optional - Force drop of database
#
gr_create_database()
{
new_db=$1
luser=$2
lpass=$3
drop_db=$4

sql_db_already_exists="SQL1005N"
err_db2_not_found=42705		# Sql code for db not found
err_db_not_started=57019	# Sql code for db not running
err_db_no_catalog=1005		# Sql code for db needs catalog

echo "==> Finding/Creating database ($new_db)"

if [ "$drop_db" = "" ]; then
    drop_db=$FALSE
fi

# Force drop the db if requested
if [ "$drop_db" -eq $TRUE  ]; then
  echo "      (I) Dropping DB as requested"
  blah=`$DB2EXEC_LOG drop db $new_db `
  :;		# throw away error if there is one
fi


# Find/Create the Database
blah=`$DB2EXEC_LOG connect to "$new_db" user "$luser" using "$lpass"`
if [ $? -ne 0 ]; then
  sql_err_code=`echo $blah | sed "s/.*SQLSTATE=//" `
  if [ "$sql_err_code" = "$err_db2_not_found" ]; then
    # If not there, try to create it, but check space first
    echo "      (I) Checking disk space"
    gr_check_disk_usage  40000 "$DB2_INST_DIR";    # 40 meg reasonable

    # Uncatalog it just in case it exists in the catalog
    blah=`$DB2EXEC_LOG uncatalog db "$new_db" `

    # Clear any possible connections
    blah=`$DB2EXEC_LOG terminate `

    # Create the DB
    echo "      (I) Database ($new_db) not found.  Creating..."
    blah=`$DB2EXEC_LOG create database "$new_db" `
    rc=$?

    blah2=`echo $blah | grep "$sql_db_already_exists" `
    sql_rc=$?

    #
    if [ $rc -eq 0 ]; then
      # Now test a connection to it
      blah=`$DB2EXEC_LOG connect to "$new_db" user "$luser" using "$lpass" `
      if [ $? = 0 ]; then
        echo "      (I) Connection successful to newly created database ($new_db)"
	else
        echo "      (F) CDWIC0015E Failed to connect to new database.  Exiting"
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	  exit 15
      fi
    # The db exists locally, but not in our public catalog.  Catalog it.
    elif [ $rc -ne 0 -a $sql_rc -eq 0 ]; then
      echo "      (I) Database detected on this system, but not publicly cataloged.  Cataloging..."
      blah=`$DB2EXEC_LOG catalog database "$new_db" authentication server `
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0015E Failed to catalog local database ($new_db)."
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
        exit 15
      fi

      # now connect test it
      blah=`$DB2EXEC_LOG connect to "$new_db" user "$luser" using "$lpass" `
      if [ $? = 0 ]; then
        echo "      (I) Connection successful to newly cataloged database ($new_db)"
    else
        echo "      (F) CDWIC0015E Failed to connect to newly cataloged database.  Exiting"
      echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	  exit 15
    fi
    else
      # It might not be started, start it.
      echo "      (I) Starting the database instance..."
      blah=`$DB2EXEC_LOG db2start `
      # We assume this works and find out later if it doesn't.
      # This gets around the synch manager log corruption problem
      # that can return a non zero return code even if db2 does start
      # if [ $? -ne 0 ]; then
      #   echo "      (F) CDWIC0030E Could not start existing DB ($new_db)"
      #   exit 30
      # fi

      # Try creating again...
      echo "      (I) Now creating the database..."
      blah=`$DB2EXEC_LOG create database "$new_db" `
      if [ $? -ne 0 ]; then
        blah=`$DB2EXEC_LOG catalog database "$new_db" authentication server `
      fi

      # now connect test it
      echo "      (I) Testing the connection"
      blah=`$DB2EXEC_LOG connect to "$new_db" user "$luser" using "$lpass" `
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0016E Failed trying to create the database ($new_db)"
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	  exit 16
      fi
    fi
  # Database not started.  Start it.
  elif [ "$sql_err_code" = "$err_db_not_started" ]; then
    echo "      (I) Found Database, Instance not running, Starting..."
      blah=`$DB2EXEC_LOG db2start `
      # if [ $? -ne 0 ]; then
      #   echo "      (F) CDWIC0030E Could not start existing DB ($new_db)"
      #   exit 30
      # fi
  else
    echo "      (F) CDWIC0017E Unexpected SQL error during connect test."
    echo "          SQL error code was ($sql_err_code) for ($new_db)"
    exit 17
  fi
else
  echo "      (I) Database ($new_db) detected and is accessable."
fi

# First uncatalog the ODBC data source (don't care about ret code here)
echo "      (I) Cataloging the ODBC Database datasource"
upcase_db=`echo $new_db | tr '[a-z]' '[A-Z]' `
blah=`$DB2EXEC_LOG uncatalog system odbc data source "$upcase_db" `

# now catalog
blah=`$DB2EXEC_LOG catalog system odbc data source "$upcase_db" `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIC0018E Could not catalog DB ($upcase_db)"
  exit 18
fi


# Tag file to tell us what type of install was selected
# Prevents DB issues
if [ $CUSTOM_INSTALL -eq $FALSE ]; then
  touch "$CORE_INSTALL_TYPICAL_FILE"
else
  touch "$CORE_INSTALL_CUSTOM_FILE"
fi


return 0;
}



###


# Generic remote database creation function
# To call:  gr_create_remote_database $1;
#           $1 = DB name to create
#		$2 = Remote Host name (fully qualified)
#		$3 = Alias node name (generated)
#		$4 = Remote Port db server listening
#		$5 = Remote User
#		$6 = Report Password
#		$7 = Force drop of db first (Optional)
#
gr_create_remote_database()
{
new_db="$1"
rhost="$2"
rnode_alias="$3"
rport="$4"
ruser="$5"
rpass="$6"
drop_db="$7"

sql_node_already_cataloged="SQL1018N"
sql_db_already_cataloged="SQL1005N"
sql_db_already_exists="SQL1005N"
sql_db_catalog_only_on_server="SQL1013N"
err_db2_not_found=42705		# Sql code for db not found
err_db_not_started=57019	# Sql code for db not running
tmp_attach_file="$TMPDIR/tmp_twh_remote_db_create.sql"

echo "==> Finding/Creating remote database ($new_db) on ($rhost)"

if [ "$drop_db" = "" ]; then
    drop_db=$FALSE
fi

# Force drop the db if requested
if [ "$drop_db" -eq $TRUE  ]; then
  echo "      (I) Dropping DB as requested"
  blah=`$DB2EXEC_LOG drop db $new_db `
  :;		# throw away error if there is one
fi

sql_create_remote="attach to "$rnode_alias" user "$ruser" using "$rpass";
create database "$new_db";
detach;"

# Catalog the remote node
blah=`$DB2EXEC_LOG catalog tcpip node "$rnode_alias" remote "$rhost" server "$rport" `
if [ $? -ne 0 ]; then
  # Ignore if it is already cataloged
  blah2=`echo $blah | grep "$sql_node_already_cataloged" `
  if [ $? -ne 0 ]; then
    echo "      (F) CDWIC0031E Failed to catalog remote node for CDW.  Node is ($rhost)"
    exit 31
  else
    echo "      (I) Node ($rhost) already cataloged"
  fi
fi

# Clear any possible connections
blah=`$DB2EXEC_LOG terminate `


# Create the remote DB
blah=`$DB2EXEC_LOG connect to "$new_db" user "$ruser" using "$rpass"`
if [ $? -ne 0 ]; then
  sql_err_code=`echo $blah | sed "s/.*SQLSTATE=//" `
  if [ "$sql_err_code" = "$err_db2_not_found" ]; then
    # If not there, try to create it
    echo "      (I) Cannot connect to database ($new_db)."
    echo "          Attempting to Create..."

    # Drop any local catalog to prevent problems later, ignore errors
    blah=`$DB2EXEC_LOG uncatalog database "$new_db" `

    # Clear any possible connections second time, clears cache and bp
    blah=`$DB2EXEC_LOG terminate `

    # *** Create DB ***
    echo "$sql_create_remote" > "$tmp_attach_file"
    create_out=`$DB2EXEC_LOG_DDL \"$tmp_attach_file\" `
    rc=$?

    # Clean up tmp file with user and pass
    if [ -z "$TWH_KEEP_TMP_FILES_TRUE" ]; then
      rm -f "$tmp_attach_file"
    else
      gr_clean_conn_string "$tmp_attach_file" "$TMPDIR";
    fi

    blah2=`echo $create_out | grep "$sql_db_already_exists" `
    sql_rc=$?

    #
    if [ $rc -eq 0 ]; then
      echo "      (I) Database created."
      # Now test a connection to it
      blah=`$DB2EXEC_LOG connect to "$new_db" user "$ruser" using "$rpass" `
      if [ $? -eq 0 ]; then
        echo "      (I) Connection successful to newly created database ($new_db)"
	else
        echo "      (F) CDWIC0015E Failed to connect to new database.  Exiting"
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	  exit 15
      fi
    # The remote db exists, but not in our local catalog.  Catalog it.
    elif [ $rc -ne 0 -a $sql_rc -eq 0 ]; then
      echo "      (I) Remote Database detected, but not locally cataloged.  Cataloging..."
      blah=`$DB2EXEC_LOG catalog database "$new_db" at node "$rnode_alias" authentication server `
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0015E Failed to catalog remote database ($new_db)."
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
        exit 15
      fi

      echo "      (I) Database cataloged sucessfully"
      echo "          Now testing connection"

      # now connect test it
      conn_out=`$DB2EXEC_LOG connect to "$new_db" user "$ruser" using "$rpass" `
      rc=$?

      # Check to see if the database is cataloged on the server, but
	# the actual database does not exist.  Can't fix this.
      blah=`echo $conn_out | grep "$sql_db_catalog_only_on_server" `
      sql_rc=$?

      if [ $rc -eq 0 ]; then
        echo "      (I) Connection successful to newly cataloged database ($new_db)"
      elif [ $rc -ne 0 -a $sql_rc -eq 0 ]; then
        echo "      (F) CDWIC0015E Error trying to connect to remote database on host ($rhost)."
        echo "          The database may be cataloged, but not exist."
        echo "          Cannot recover from this type of error."
        echo "          Uncatalog the remote catalog for ($new_db)"
        echo "          on the database server host."
        echo "          DB2 Command is:  'Uncatalog database $new_db'"
        echo "          Then retry the installation."
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
        exit 15
	else
        echo "      (F) CDWIC0015E Failed to connect to newly cataloged database.  Exiting"
        echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	  exit 15
      fi
    else
      echo "      (F) CDWIC0016E Failed to create database ($new_db) on ($rhost)"
      echo "          Look in log ($TMPDIR/$DB2_INSTALL_LOG) for specific error information"
	exit 16
    fi
  elif [ "$sql_err_code" = "$err_db_not_started" ]; then
    echo "      (I) Found database, instance not running, Starting..."
      blah=`$DB2EXEC_LOG db2start `
      # if [ $? -ne 0 ]; then
      #   echo "      (F) CDWIC0030E Could not start existing DB ($new_db)"
      #   exit 30
      # fi
  else
    echo "      (F) CDWIC0032E Could not connect to database ($new_db)."
    echo "          Check supplied values like username and password."
    exit 32
  fi
else
  echo "      (I) Remote database ($new_db) detected and is accessable."
fi

# Tag file to tell us what type of install was selected
# Prevents DB issues
if [ $CUSTOM_INSTALL -eq $FALSE ]; then
  touch "$CORE_INSTALL_TYPICAL_FILE"
else
  touch "$CORE_INSTALL_CUSTOM_FILE"
fi

return 0;
}


###


# This function will find the script file and fix up the name a bit
# Call as: gr_build_script_name $1 $2
#       $1 = script to build
#       $2 = path to find script
#       Sets the variable NAMMED_SCRIPT to be assigned by caller
#
gr_build_script_name()
{
gen_script="$1"
gen_script_path="$2"
NAMMED_SCRIPT=""

# Test input
if [ -z "$gen_script" ]; then
  echo "      (F) CDWIC0019E Could not locate first parameter for build_script_name()"
  exit 19
fi

if [ -z "$gen_script_path" ]; then
  echo "      (F) CDWIC0019E Could not locate second parameter for build_script_name()"
  exit 19
fi


# Find and fixup the name of the script
orig_script="$gen_script"
NAMMED_SCRIPT=`ls "$gen_script_path"/$gen_script 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Could not locate the ddl script"
  echo "          Script is ($exec_dir/$orig_script)."
  echo "          Asterisk represents the app short name."
  NAMMED_SCRIPT=""
  return 0
fi
NAMMED_SCRIPT=`basename "$NAMMED_SCRIPT" `

#echo "script is: $NAMMED_SCRIPT"

return 0;
}


###


# Build a ddl script we can run from the command line
# To call:	gr_build_exec_script $1 $2 $3 ($4);
#	$1 = Database name
#	$2 = DDL/DML script location
#	$3 = DDL/DML script file
#	$4 = DB2 Type (MD, CDW, MART)
#	$5 = Connection type (Connect or Attach)
#	$6 = SQL line delimiter (Optional)
gr_build_exec_script()
{
db_name="$1"
script_location="$2"
script_file="$3"
db_type="$4"
conn_type=$5
delimeter="$6"
TMP_DB_SCRIPT_FILE="$TMPDIR/tmp_$script_file"


# Set the user and pass based on install type
if [ $CUSTOM_INSTALL -eq $FALSE ]; then
  db_user="$DB2USER"
  db_pass="$DB2PASS"
  conn_host=`hostname `
elif [ $db_type -eq $COPT_CDW_TYPE ]; then
  db_user="$COPT_CDW_DB2USER"
  db_pass="$COPT_CDW_DB2PASS"

  if [ $conn_type -eq $CONN_TYPE_ATTACH ]; then
    gr_get_node_alias $CDW_ALIAS_TYPE;
    conn_host="$GR_NODE_ALIAS"
  fi
elif [ $db_type -eq $COPT_MART_TYPE ]; then
  db_user="$COPT_MART_DB2USER"
  db_pass="$COPT_MART_DB2PASS"

  if [ $conn_type -eq $CONN_TYPE_ATTACH ]; then
    gr_get_node_alias $MART_ALIAS_TYPE;
    conn_host="$GR_NODE_ALIAS"
  fi
elif [ $db_type -eq $COPT_CTRL_TYPE ]; then
  db_user="$COPT_CTRL_DB2USER"
  db_pass="$COPT_CTRL_DB2PASS"

  if [ $conn_type -eq $CONN_TYPE_ATTACH ]; then
    gr_get_node_alias $CTRL_ALIAS_TYPE;
    conn_host="$GR_NODE_ALIAS"
  fi
else
  echo "      (F) CDWIC0033E Unknown Custom option component type"
  exit 33
fi


# Set the connect string to Connect or Attach
if [ $conn_type -eq $CONN_TYPE_CONNECT ]; then
  conn_string="connect to "$db_name" user "$db_user" using "$db_pass""
elif [ $conn_type -eq $CONN_TYPE_ATTACH ]; then
  conn_string="attach to "$conn_host" user "$db_user" using "$db_pass""
else
  echo "      (F) CDWIC0034E Unknown connection type."
  exit 34
fi

# Set default delimeter
if [ "$delimeter" = "" ]; then
    delimeter=";"
fi

echo "      (I) Building DDL script for ($script_file)"

# Create the new ddl script with connect db, user, and pass string in it
if [ ! -r "$script_location/$script_file" ]; then
  echo "      (F) CDWIC0020E Could not locate ddl file ($script_location/$script_file)."
  exit 20
fi

# Make new tmp SQL exec script
echo $conn_string > "$TMP_DB_SCRIPT_FILE"
echo "$delimeter" >> "$TMP_DB_SCRIPT_FILE"

cat "$script_location/$script_file" >> "$TMP_DB_SCRIPT_FILE"

if [ ! -r "$TMP_DB_SCRIPT_FILE" ]; then
  echo "      (F) CDWIC0021E Error trying to read Created DDL execution script"
  rm -f "$TMP_DB_SCRIPT_FILE"
  exit 21
elif [ -x "$TMP_DB_SCRIPT_FILE" ]; then
  chmod 777 "$TMP_DB_SCRIPT_FILE"
fi

echo "      (I) DDL script prepared"
return 0;
}


###


# Execute the DDL/DML SQL script
# To call:	gr_exec_sql_script $1 $2 ($3);
#	$1 = Database name
#	$2 = Trap SQL Errors
#	$3 = (Optional) delimeter for SQL statements
gr_exec_sql_script()
{
db_name="$1"
trap_error=$2
delimeter="$3"

echo "      (I) ** Executing SQL script ($TMP_DB_SCRIPT_FILE)"
echo "             on database ($db_name)"

# Is it really there?
if [ ! -r "$TMP_DB_SCRIPT_FILE" ]; then
  echo "      (F) CDWIC0021E Error trying to read Created DDL execution script"
  exit 21
fi


# Execute the SQL script
# Must support different delimeters
if [ "$delimeter" = "" ]; then
  emsg=`$DB2EXEC_LOG_DDL \"$TMP_DB_SCRIPT_FILE\" 2>&1 `
else
  emsg=`$DB2EXEC_LOG -td"$delimeter" -vf \"$TMP_DB_SCRIPT_FILE\" 2>&1`
fi

rc=$?
# Under a retcode of 3 by DB2 are warnings - ignore here
if [ $rc -ge 3 -a $trap_error -eq $TRUE ]; then
  echo "      (F) CDWIC0022E Failure detected during SQL script execution"
  echo "          for Database ($db_name)."
  echo "          Return code was ($rc)"
  echo "          Check messages in DB2 log ($DB2_INSTALL_LOG)."
  if [ -z "$TWH_KEEP_TMP_FILES_TRUE" ]; then
    echo "          Removing TMP script."
    rm -f "$TMP_DB_SCRIPT_FILE"
  else
    gr_clean_conn_string "$TMP_DB_SCRIPT_FILE" "$TMPDIR";
  fi
  echo "          Exiting..."
  exit 22
# Ignore error if instructed
elif [ $rc -ge 3 -a $trap_error -ne $TRUE ]; then
  echo "      (W) Error detected, but instructed to continue."
  # But clean out conn string
  gr_clean_conn_string "$TMP_DB_SCRIPT_FILE" "$TMPDIR";
  WARNING_FLAG=$TRUE
# Default rule is to always ignore warnings, but tell them.
elif [ $rc -gt 0 -a $rc -lt 3 ]; then
  echo "      (W) Warnings detected during run of SQL script."
  echo "          Can check messages in DB2 log"
  echo "          ($DB2_INSTALL_LOG)."
  echo "          Continuing"
  gr_clean_conn_string "$TMP_DB_SCRIPT_FILE" "$TMPDIR";
  WARNING_FLAG=$TRUE
fi

fname=`basename "$TMP_DB_SCRIPT_FILE" `

# Remove tmp file if requested
if [ -z "$TWH_KEEP_TMP_FILES_TRUE" ]; then
  rm -f "$TMP_DB_SCRIPT_FILE"
elif [ -n "$TWH_KEEP_TMP_FILES_TRUE" -a -n "$TWH_APP_CONTEXT" ]; then
  gr_clean_conn_string "$TMP_DB_SCRIPT_FILE" "$APP_LOGDIR/$fname"
elif [ -n "$TWH_KEEP_TMP_FILES_TRUE" -a -n "$TWH_APP_CONTEXT" ]; then
  gr_clean_conn_string "$TMP_DB_SCRIPT_FILE" "$TWH_TOPDIR/install/logs/$fname"  
fi


echo "      (I) SQL script completed"
echo ""
return 0;
}


###


# This routine clean the connect string from a file
# to ensure the user and pass are not found.
#	$1 = Name SQL of file to clean
#	$2 = The path to the place to move the files
gr_clean_conn_string()
{
orig_file="$1"
to_path=$2

if [ -z "$orig_file" ]; then
  echo "      (F) CDWIC0023E Error trying to clean SQL file.  No supplied file value to clean"
  exit 23
fi


fname=`basename "$orig_file"`
cat "$orig_file" | sed "s/connect.*/--/" > "$TMPDIR"/clean_$fname
rm -f "$orig_file"
mv "$TMPDIR"/clean_$fname "$to_path"/$fname


return 0;
}


###


# This bogus statement will mark each attempt on install in the log
# Look for this bogus error in the DB2 log to see where things started
# We expect this to fail.  Look for 'MARKsomething' in DB2 log.  Value
# might be CORE or $AVA code.
#     $1 = Is 'MARK' then either CORE or the ava_code.
gr_mark_db2_log_for_install()
{
marker=$1
echo "      (I) Marking DB2 Log"


if [ $UNIX_INTERP -eq 1 ]; then
  gr_su_cmd $DB2USER "$DB2EXEC_LOG_U connect to $marker user bogus using bogus1 2>&1"
else
  # NT OS
  blah=`$DB2EXEC_LOG connect to $marker user bogus using bogus1 2>&1 `
  if [ $? != 0 ]; then :; fi	# Throw this error away
fi

return 0;
}


###


# Generic su function.  Needed since su on Linux is different than su on
# other UNIXs.  Linux su will allways return a 0 return code no matter
# what is the return code of the executed command is.
#
# $1 - username to su to
# $2 - command to run
#
# This function will set the GR_SU_OUT environment variable with the
# output that is returned from the su command.  The return code for
# this function is the return code from the executed command.
gr_su_cmd()
{
user_to_su=$1
cmd_to_run=$2

blah=`su - $user_to_su -c "$cmd_to_run;echo \\\$?"`
GRSU_OUT=`echo $blah | awk {' for (n=1; n<NF; n++) print $n '}`
rc=`echo $blah | awk {' print $(NF) '}`
return $rc
}


###


# Make sure we can access DB2 prog (checks PATH)
gr_test_db2exec()
{
echo "==> Testing DB2 exec path"


if [ $UNIX_INTERP -eq 1 ]; then
  DB2EXEC_LOG_U="db2 -z \"$TMPDIR/$DB2_INSTALL_LOG\" "
  DB2EXEC_LOG_DDL_U="$DB2EXEC_LOG_U -tvf "


  # Test to see if the user they gave us is on the system
  echo "      (I) Testing DB2 user ($DB2USER)"
  su - $DB2USER -c "echo '          ...User test Valid.'"
  if [ $? -ne 0 ]; then
    echo "      (F) Invalid DB2 user supplied or user not on system."
    exit 24
  fi


  # Now test db2 command
  echo "      (I) Testing DB2 command."
  gr_su_cmd $DB2USER "$DB2EXEC_LOG_U bogus_command 2>&1"
  rc=$?

  err_txt="$GRSU_OUT"
  if [ $rc -ne 4 ]; then
    echo "      (F) CDWIC0024E Could not execute/locate DB2 command!!!"
    echo "          May have a problem with the DB2 installation,"
    echo "          command permissions, or PATH"
    echo "          Error is:"
    echo "          ($err_txt)"
    exit 24
  fi
else
  # NT OS
  blah=`$DB2EXEC bogus_command 2>&1 `
  rc=$?
  if [ $rc -ne 4 ]; then
    echo "      (F) CDWIC0024E Could not execute/locate DB2 command!!!"
    echo "          Return code is ($rc) and error is:"
    echo "          $blah"
    exit 24
  fi
fi


echo "      (I) DB2 exec path completed"
return 0;
}


###


# Test connection.  Start db2 if detected to no be running
gr_test_connection()
{
db="$1"
dbuser="$2"
dbpass="$3"
err_db_not_started=57019	# Sql code for db not running

echo "      (I) Testing connection to DB"


if [ $UNIX_INTERP -eq 1 ]; then
  gr_su_cmd $DB2USER "$DB2EXEC_LOG_U connect to $db user $dbuser using $dbpass"
  if [ $? -ne 0 ]; then
    sql_err_code=`echo $GRSU_OUT | sed "s/.*SQLSTATE=//" `
    if [ "$sql_err_code" = "$err_db_not_started" ]; then
      echo "      (I) Found database, instance not running, trying to Start..."
      gr_su_cmd $DB2USER "$DB2EXEC_LOG_U db2start"
      # if [ $? -ne 0 ]; then
      #   echo "      (F) CDWIC0030E Could not start existing DB ($db)"
      #   exit 30
      # fi
    else
      echo "      (F) CDWIC0032E Could not connect to ($db) DB."
      exit 32
    fi
  else
    echo "      (I) Connection accepted"
  fi
else
  # Check to see if DB2 started
  blah=`$DB2EXEC_LOG connect to "$db" user "$dbuser" using "$dbpass" `
  if [ $? -ne 0 ]; then
    sql_err_code=`echo $blah | sed "s/.*SQLSTATE=//" `
    if [ "$sql_err_code" = "$err_db_not_started" ]; then
      echo "      (I) Found database, instance not running, trying to Start..."
      blah=`$DB2EXEC_LOG db2start `
      # if [ $? -ne 0 ]; then
      #   echo "      (F) CDWIC0030E Could not start existing DB ($db)"
      #   exit 30
      # fi
    else
      echo "      (F) CDWIC0032E Could not connect to ($db) DB."
      exit 32
    fi
  else
    echo "      (I) Connection accepted"
  fi
fi

return 0;
}


###


#
gr_display_return_codes()
{
echo "    0 = Successful completion"
echo "    1 = Warnings were detected.  Review supporting logs"
echo "    2 = General error detected."
echo "    3 = Unable to read or locate the TEDW Config file."
echo "    4 = Not all required command line options were supplied."
echo "    5 = A property was detected as missing from the Config file"
echo "    6 = No Context (Core or App) set for generic routines"
echo "    7 = An illegal value was found in the Config file"
echo "    8 = Unsupported Interp type supplied"
echo "    9 = Disk space calculation error"
echo "    10 = Tmp Directory not accessible"
echo "    11 = Error extending path for Unix tools"
echo "    12 = Invalid User ID for installation process"
echo "    13 = Reinstallation error.  Cannot install on current version"
echo "    14 = Could not create directory in the TEDW Install tree"
echo "    15 = Failed to connect to created database"
echo "    16 = Could not create new database as requested"
echo "    17 = Unexpected SQL error during connect test"
echo "    18 = Could not catalog new database"
echo "    19 = Invalid function call.  Missing arguments"
echo "    20 = Could not locate SQL script to execute"
echo "    21 = Could not read/access SQL script"
echo "    22 = Failure during SQL script execution.  Check DB2 SQL log"
echo "    23 = Error trying to scrub User and Password from SQL script"
echo "    24 = Could not execute test DB2 command"

return 0;
}


###


# Determine if the host they supplied is on the current host
#	$1 = The name of the host to check against (must be fully qualified)
#	returns:	True or False
gr_on_host()
{
echo "      (I) Testing if on local host"
passed_hostname="$1"

if [ -z "$passed_hostname" ]; then
  echo "      (F) CDWIC0035E Did not supply a hostname to determine if resident"
  exit 35
fi

# convert to upper for consistency
passed_hostname=`echo $passed_hostname |  tr '[a-z]' '[A-Z]' `
full_hname=`echo $LOCAL_HOSTNAME |  tr '[a-z]' '[A-Z]' `

echo "          Supplied host is ($passed_hostname)"
echo "          Local hostname is ($full_hname)"

if [ "$full_hname" = "$passed_hostname" ]; then
  echo "      (I) Host specified is local"
  return $TRUE;
fi

echo "      (I) Host is remote"
return $FALSE;
}


###


# This routine will trim any of the .dev.blah behind the hostname
#	$1 = The host to trim
# 	The returned value is supplied in SHORT_HNAME var
gr_host_shortname()
{
passed_hname="$1"

if [ -z "$passed_hname" ]; then
  echo "      (F) CDWIC0035E Did not supply a hostname to trim"
  exit 35
fi

HNAME_SHORT=`echo $passed_hname | sed "s/\..*//g" `

return 0;
}


###


# Validate the Custom Option Parameters
#	$1 = Type of Custom Option (cdw or mart)
gr_validate_copts()
{
type="$1"

echo "      (I) Validating values for Custom CDW Installation"

if [ $CUSTOM_INSTALL -eq $FALSE ]; then
  echo "      (F) CDWIC0036E The call to this function is invalid."
  echo "          Must be under a custom installation."
  exit 36
fi

if [ $type -eq $COPT_CDW_TYPE ]; then
  copt=`echo "$COPT_CDW_DB2HOST" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_CDW_DB2HOST cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_CDW_DB2PORT" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_CDW_DB2PORT cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_CDW_DB2USER" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_CDW_DB2USER cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_CDW_DB2PASS" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_CDW_DB2PASS cannot be NULL during custom install"
    exit 37
  fi
elif [ $type -eq $COPT_MART_TYPE ]; then
  copt=`echo "$COPT_MART_DB2HOST" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_MART_DB2HOST cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_MART_DB2PORT" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E  The value for COPT_MART_DB2PORT cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_MART_DB2USER" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_MART_DB2USER cannot be NULL during custom install"
    exit 37
  fi

  copt=`echo "$COPT_MART_DB2PASS" | tr '[a-z]' '[A-Z]' `
  if [ "$copt" = "NULL" ]; then
    echo "      (F) CDWIC0037E The value for COPT_MART_DB2PASS cannot be NULL during custom install"
    exit 37
  fi
else
  echo "      (F) CDWIC0033E Unknown Custom Option type"
  exit 33
fi

return 0;
}


###


# Catalog the DB's
# Because we can get so many different errors back from DB2
# when we try to connect, just try some basic steps to
# get going and it if still does not connect return
# the original db2 connect error.
gr_catalog_db()
{
db="$1"
dbhost="$2"
dbport="$3"
dbuser="$4"
dbpass="$5"
err_db_not_found=42705		# Sql code for db not found
err_db_not_started=57019	# Sql code for db not running
err_node_not_found="42720"	# Sql code for node not found
err_db_already_catalog=1005	# Sql code for db needs catalog
err_db_no_catalog_entries="58031"	# No entries in the db catalog
err_db_already_catalog_u="SQL1005N"
err_node_already_cataloged="1018"	# Sql code for node cataloged
err_node_already_cataloged_u="SQL1018N"


echo "==> Cataloging Database ($db)"

gr_on_host "$dbhost";
local_to_host=$?


# determine the aliased node name for DB2
if [ $local_to_host -eq $FALSE ]; then
  if [ "$db" = "$TWH_CTRL_DB" ]; then
    get_node_alias $CTRL_ALIAS_TYPE;
    dbhost_short="$GR_NODE_ALIAS"
  elif [ "$db" = "$TWH_CDW_DB" ]; then
    get_node_alias $CDW_ALIAS_TYPE;
    dbhost_short="$GR_NODE_ALIAS"
  elif [ "$db" = "$TWH_DM_DB" ]; then
    get_node_alias $MART_ALIAS_TYPE;
    dbhost_short="$GR_NODE_ALIAS"
  fi
fi


if [ $UNIX_INTERP -eq 1 ]; then
  # Unix does not have a catalog system ODBC data source cli. They
  # are files.  But we don't need to do that

  # Can do a connect test first to see if there
  gr_su_cmd $DB2USER "$DB2EXEC_LOG_U connect to $db user $dbuser using $dbpass"
  if [ $? -ne 0 ]; then
    stored_sql_error="$GRSU_OUT"
     
      # Try to catalog it
      if [ $local_to_host -ne $TRUE ]; then
        gr_su_cmd $DB2USER "$DB2EXEC_LOG_U catalog tcpip node $dbhost_short remote $dbhost server $dbport 2>&1" 
        if [ $? -ne 0 ]; then
	  sql_err_code=`echo $GRSU_OUT | grep $err_node_already_cataloged_u ` 
          if [ $? -eq 0 ]; then
	    echo "      (I) Node catalog found."
	  else
            echo "      (F) CDWIC0038E Could not catalog tcpip node ($dbhost)"
            exit 38
	  fi
	else
	  echo "      (I) Node catalog added for ($dbhost_short)."
        fi
      fi

      remote_txt=""
      if [ $local_to_host -ne $TRUE ]; then
        remote_txt="at node $dbhost_short authentication server"
      fi

      # Catalog the DB
      gr_su_cmd $DB2USER "$DB2EXEC_LOG_U catalog database $db $remote_txt 2>&1"
      if [ $? -ne 0 ]; then
	sql_err_code=`echo $GRSU_OUT | grep $err_db_already_catalog_u `
	if [ $? -eq 0 ]; then
	  echo "      (I) DB catalog found."
	else
          echo "      (F) CDWIC0039E Could not catalog database ($db)"
          exit 39
	fi
      else
        echo "      (I) DB catalog created."
      fi

      # Connect test to verify new cataloged db
      echo "      (I) Trying to connect to new catalog of db"
      gr_su_cmd $DB2USER "$DB2EXEC_LOG_U connect to $db user $dbuser using $dbpass"
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0032E Could not connect to database named ($db)"
        exit 32
      fi
  else
    echo "      (I) Connected to existing cataloged Database"
  fi
else
  # NT OS
  # Test connect to db
  blah=`$DB2EXEC_LOG connect to $db user $dbuser using $dbpass `
  if [ $? -ne 0 ]; then
  stored_sql_error="$blah"

      # If it failed, try to catalog it (node first, then db)
      if [ $local_to_host -ne $TRUE ]; then
        blah=`$DB2EXEC_LOG catalog tcpip node $dbhost_short remote $dbhost server $dbport` 
	if [ $? -ne 0 ]; then
 	  sql_err_code=`echo $blah | grep $err_node_already_cataloged `
          if [ $? -eq 0 ]; then
	    echo "      (I) Node catalog found."
	  else
            echo "      (F) CDWIC0038E Could not catalog tcpip node ($dbhost)"
            exit 38
	  fi
	else
	  echo "      (I) Node catalog added for ($dbhost_short)."
        fi
      fi

      remote_txt=""
      if [ $local_to_host -ne $TRUE ]; then
        remote_txt="at node $dbhost_short authentication server"
      fi

      # Catalog the db
      blah=`$DB2EXEC_LOG catalog database $db $remote_txt `
      if [ $? -ne 0 ]; then
	sql_err_code=`echo $blah | grep $err_db_already_catalog `
	if [ $? -eq 0 ]; then
	  echo "      (I) DB catalog found."
	else
          echo "      (F) CDWIC0039E Could not catalog database ($db)"
          exit 39
	fi
      else
        echo "      (I) DB catalog created."
      fi


      # Catalog all created DB's even if already created (NT only)
      echo "      (I) Cataloging the DB"
      upcase_db=`echo $db | tr '[a-z]' '[A-Z]' `
      blah=`$DB2EXEC_LOG catalog system odbc data source "$upcase_db" `
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0018E Could not catalog DB ($upcase_db)"
        exit 18
      fi

      # Connect to new cataloged DB
      echo "      (I) Trying to connect to new catalog of db"
      blah=`$DB2EXEC_LOG connect to $db user $dbuser using $dbpass `
      if [ $? -ne 0 ]; then
        echo "      (F) CDWIC0032E Could not connect to new catalog of database named  ($db)"
        echo "          Original connection error was:"
        echo "          $stored_sql_error"
        exit 32
      fi
  else
    echo "      (I) Connected to existing cataloged Database"
  fi
fi


echo "      (I) Completed catalog"

return 0;
}


###


# 	$1 = Interger value for the component type
gr_get_node_alias()
{
alias_type=$1
GR_NODE_ALIAS="NULL"

if [ -z "$alias_type" ]; then
  echo "      (F) CDWIC0002E No supplied value for alias_type in gr_get_node_alias"
  exit 2
fi


# Set the requested node alias
if [ $alias_type -eq $CTRL_ALIAS_TYPE ]; then
  if [ -z "$TWH_CTRL_NODE_ALIAS" ]; then
    echo "      (F) CDWIC0002W Node alias for CTRL db not found!"
    exit 2
  fi

  GR_NODE_ALIAS="$TWH_CTRL_NODE_ALIAS"
elif [ $alias_type -eq $CDW_ALIAS_TYPE ]; then
  if [ -z "$TWH_CDW_NODE_ALIAS" ]; then
    echo "      (F) CDWIC0002W Node alias for CDW db not found!"
    exit 2
  fi

  GR_NODE_ALIAS="$TWH_CDW_NODE_ALIAS"
elif [ $alias_type -eq $MART_ALIAS_TYPE ]; then
  if [ -z "$TWH_MART_NODE_ALIAS" ]; then
    echo "      (F) CDWIC0002W Node alias for MART db not found!"
    exit 2
  fi

  GR_NODE_ALIAS="$TWH_MART_NODE_ALIAS"
fi


# make it public
export GR_NODE_ALIAS

echo "      (I) Node alias is $GR_NODE_ALIAS"
return 0;
}
