#!/bin/sh
##############################################################################
##
## Tivoli Enterprise Data Warehouse, v 1.1.0 
## (C) Copyright IBM Corp. 2002.  All Rights Reserved.
##
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with
## IBM Corp.
##
##
##############################################################################
##
## File Information: src/tools/bin/rollup.sh, twh_core, twh_1.1_fp1
##
## Revision: 1.7
##
## Date: 5/3/02 12:44:22
## 
## Description: Called as a UDP from the DWC to aggregate hourly data into 
##              daily, weekly and monthly tables.
## Inputs:
## 1. name of schema
## 2. name of hour fact table
## 3. name of stage table used to populate hour fact table
## 4-6. dbname, user, pwd of twh_mart
## 7-9. dbname, user, pwd of twh_md
## 10. stepname (this is used to write the log file)
## Function:
## 1. derive name of each star schema (hour/day/week/month) and fact table
## 2. make runtime rollup sqlscript based on rollup template
## 3. run the rollup sqlscript
##
## Original Author: J.Cook
##
## Current Owner: K. Henley
##
##############################################################################

SCHEMA=$1
HOURFACT=$2
STAGE=$3
MARTDB=$4
MARTUSER=$5
MARTPWD=$6
CNTLDB=$7
CNTLUSER=$8
CNTLPWD=$9
STEPNAME=${10}

# DB2 program variables
DB2CMD="db2cmd /w /c /i"
DB2PROG="db2"						# DB2 interface command
DB2EXEC="$DB2CMD $DB2PROG"				# Execute db2 command

VWS_LOGGING=` echo "$VWS_LOGGING" | sed 's/\\\\/\//g' `

# Variables used to run sql commands as temporary scripts
connectString="connect to $CNTLDB user $CNTLUSER using $CNTLPWD"
tempSqlScript=""$VWS_LOGGING"/"$STEPNAME"_temp.sql"
tempSqlOutput=""$VWS_LOGGING"/"$STEPNAME"_temp1.out"
tempSqlOutput2=""$VWS_LOGGING"/"$STEPNAME"_temp2.out"
diagOutput=""$VWS_LOGGING"/"$STEPNAME"_diag"
delimeter=";"

echo "ARGS PASSED: $1, $2, $3, $4, $5, $6, $7, $8, $9, ${10}" > "$diagOutput"

# In case the last run got an error...remove old output file that
# would be appened to by DB2.
rm "$tempSqlOutput"

# get the join columns for this star schema identified by its hour fact table
echo "$connectString" > "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"
echo "select rtrim(colname) concat ',' from rpi.ss_joincols where tabname = '$HOURFACT' and schema = '$SCHEMA'" >> "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"

errOut=` $DB2EXEC -sxtd\"$delimeter\" -z \"$tempSqlOutput\" -f \"$tempSqlScript\" `
if [ $? -ne 0 ]; then
   echo "<MSG>CDW8101E Select from rpi.ss_joincols failed with the following message: $errOut </MSG>" >> "$VWP_LOG"
   echo "<RC>8101</RC>" >> "$VWP_LOG"
   exit 8101
fi

# Remove the first 8 lines (the connect info) then the newlines and extra spaces
JOINCOLS=` cat "$tempSqlOutput" | sed '1,8d' | tr -d '\12' | tr -s ' ' ' ' ` 

rm "$tempSqlOutput"

echo "Joincols: $JOINCOLS" >> "$diagOutput"

# get the star schema id for hour

echo "$connectString" > "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"
echo "select distinct iwhid from iwh.starschema s, rpi.ss_joincols j where s.name=j.ssname and j.tabname = '$HOURFACT' and j.schema = '$SCHEMA'" >> "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"

errOut=` $DB2EXEC -sxtd\"$delimeter\" -z \"$tempSqlOutput\" -f \"$tempSqlScript\" `
if [ $? -ne 0 ]; then
   echo "<MSG>CDW8102E Select for hourly star schema failed with the following message: $errOut </MSG>" >> "$VWP_LOG"
   echo "<RC>8102</RC>" >> "$VWP_LOG"
   exit 8102
fi

HOURSSID=` cat "$tempSqlOutput" | sed '1,8d' | sed -e 's/ //g' `

numSSID=` echo "$HOURSSID" | wc -l `
if [ $numSSID -gt 1 ]; then
   echo "<MSG>CDW8106E Select for hourly star schema returned more than one star schema. The hourly fact table should only be associated with one star schema.</MSG>" >> "$VWP_LOG"
   echo "<RC>8106</RC>" >> "$VWP_LOG"
   exit 8106
fi

rm "$tempSqlOutput"

echo "Hourssid: $HOURSSID" >> "$diagOutput"

# get the star schema id for day 

DAYFACT=`echo $HOURFACT | sed -e 's/HOUR$/DAY/g' `

echo "Dayfact: $DAYFACT" >> "$diagOutput"

echo "$connectString" > "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"
echo "select distinct iwhid from iwh.starschema s, rpi.ss_joincols j where s.name=j.ssname and j.tabname = '$DAYFACT' and j.schema = '$SCHEMA'" >> "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"

errOut=` $DB2EXEC -sxtd\"$delimeter\" -z \"$tempSqlOutput\" -f \"$tempSqlScript\" `
if [ $? -ne 0 ]; then
   echo "<MSG>CDW8103E Select for daily star schema failed with the following message: $errOut </MSG>" >> "$VWP_LOG"
   echo "<RC>8103</RC>" >> "$VWP_LOG"
   exit 8103
fi

DAYSSID=` cat "$tempSqlOutput" | sed '1,8d' | sed -e 's/ //g' `

echo "Dayssid: $DAYSSID" >> "$diagOutput"

numSSID=` echo "$DAYSSID" | wc -l `
if [ $numSSID -gt 1 ]; then
   echo "<MSG>CDW8107E Select for daily star schema returned more than one star schema. The daily fact table should only be associated with one star schema.</MSG>" >> "$VWP_LOG"
   echo "<RC>8107</RC>" >> "$VWP_LOG"
   exit 8107
fi

rm "$tempSqlOutput"

# get the star schema id for week

WEEKFACT=` echo $HOURFACT | sed -e 's/HOUR$/WEEK/g' `

echo "Weekfact: $WEEKFACT" >> "$diagOutput"

echo "$connectString" > "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"
echo "select distinct iwhid from iwh.starschema s, rpi.ss_joincols j where s.name=j.ssname and j.tabname = '$WEEKFACT' and j.schema = '$SCHEMA'" >> "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"

errOut=` $DB2EXEC -sxtd\"$delimeter\" -z \"$tempSqlOutput\" -f \"$tempSqlScript\" `
if [ $? -ne 0 ]; then
   echo "<MSG>CDW8104E Select for weekly star schema failed with the following message: $errOut </MSG>" >> "$VWP_LOG"
   echo "<RC>8104</RC>" >> "$VWP_LOG"
   exit 8104
fi

WEEKSSID=` cat "$tempSqlOutput" | sed '1,8d' | sed -e 's/ //g' `

echo "Weekssid: $WEEKSSID" >> "$diagOutput"

numSSID=` echo "$WEEKSSID" | wc -l `
if [ $numSSID -gt 1 ]; then
   echo "<MSG>CDW8108E Select for weekly star schema returned more than one star schema. The weekly fact table should only be associated with one star schema.</MSG>" >> "$VWP_LOG"
   echo "<RC>8108</RC>" >> "$VWP_LOG"
   exit 8108
fi

rm "$tempSqlOutput"

# get the star schema id for month

MONTHFACT=` echo $HOURFACT | sed -e 's/HOUR$/MONTH/g' `

echo "Monthfact: $MONTHFACT" >> "$diagOutput"

echo "$connectString" > "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"
echo "select distinct iwhid from iwh.starschema s, rpi.ss_joincols j where s.name=j.ssname and j.tabname = '$MONTHFACT' and j.schema = '$SCHEMA'" >> "$tempSqlScript"
echo "$delimeter" >> "$tempSqlScript"

errOut=` $DB2EXEC -sxtd\"$delimeter\" -z \"$tempSqlOutput\" -f \"$tempSqlScript\" `
if [ $? -ne 0 ]; then
   echo "<MSG>CDW8105E Select for monthly star schema failed with the following message: $errOut </MSG>" >> "$VWP_LOG"
   echo "<RC>8105</RC>" >> "$VWP_LOG"
   exit 8105
fi

MONTHSSID=` cat "$tempSqlOutput" | sed '1,8d' | sed -e 's/ //g' `

echo "Monthssid: $MONTHSSID" >> "$diagOutput"

numSSID=` echo "$MONTHSSID" | wc -l `
if [ $numSSID -gt 1 ]; then
   echo "<MSG>CDW8109E Select for monthly star schema returned more than one star schema. The monthly fact table should only be associated with one star schema.</MSG>" >> "$VWP_LOG"
   echo "<RC>8109</RC>" >> "$VWP_LOG"
   exit 8109
fi

rm "$tempSqlOutput"

##########################################################
# replace tokens in rollup template with current values  #
##########################################################
TWH_TOPDIR=` echo "$TWH_TOPDIR" | sed 's/\\\\/\//g' `

echo "perl $TWH_TOPDIR/tools/bin/rollup.pl $TWH_TOPDIR/tools/bin/rollup.db2.template "$VWS_LOGGING"/"$STEPNAME"_rollup.db2 $SCHEMA $STAGE "$JOINCOLS" $HOURSSID $DAYSSID $WEEKSSID $MONTHSSID $HOURFACT $DAYFACT $WEEKFACT $MONTHFACT" >> "$diagOutput"

perl "$TWH_TOPDIR/tools/bin/rollup.pl" "$TWH_TOPDIR/tools/bin/rollup.db2.template" "$VWS_LOGGING"/"$STEPNAME"_rollup.db2 $SCHEMA $STAGE "$JOINCOLS" $HOURSSID $DAYSSID $WEEKSSID $MONTHSSID $HOURFACT $DAYFACT $WEEKFACT $MONTHFACT

##########################################################
# run the generated script                               #
##########################################################

execsql "$VWS_LOGGING"/"$STEPNAME"_rollup "$VWS_LOGGING"/"$STEPNAME"_rollup.log $MARTDB $MARTUSER $MARTPWD $CNTLDB $CNTLUSER $CNTLPWD

exit $?
